/*
 * Sass mixins
 */
/*
 * http://html5reset.org + minor modifications
 */
html, body, body div, span, object, iframe, h1, h2, h3, h4, h5, h6, p, blockquote, pre, abbr, address, cite, code, del, dfn, em, img, ins, kbd, q, samp, var, b, i, dl, dt, dd, ol, ul, li, fieldset, form, label, legend, table, caption, tbody, tfoot, thead, tr, th, td, article, aside, figure, footer, header, hgroup, menu, nav, section, time, mark, audio, video, details, summary {
  margin: 0;
  padding: 0;
  border: 0;
  font-size: 100%;
  line-height: 1.6em;
  vertical-align: baseline;
  background: transparent; }

article, aside, figure, footer, header, hgroup, nav, section, details, summary {
  display: block; }

ul {
  list-style: none; }

a {
  margin: 0;
  padding: 0;
  vertical-align: baseline;
  background: transparent; }

abbr[title], dfn[title] {
  border-bottom: 1px dotted #000;
  cursor: help; }

table {
  border-collapse: collapse;
  border-spacing: 0; }

th {
  font-weight: bold;
  vertical-align: bottom; }

td {
  font-weight: normal;
  vertical-align: top; }

input, select, button {
  vertical-align: middle; }

select, input, textarea {
  font-size: 14px; }

table {
  font-size: inherit;
  font: 100%; }

.clickable, label, input[type=file], button {
  cursor: pointer; }

button, input, select, textarea {
  margin: 0; }

button {
  width: auto;
  overflow: visible; }

.ie7 img {
  -ms-interpolation-mode: bicubic; }

.clearfix:before, .clearfix:after {
  content: "\0020";
  display: block;
  height: 0;
  overflow: hidden; }

.clearfix:after {
  clear: both; }

.clearfix {
  zoom: 1; }

* {
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box; }

body {
  background-color: white;
  font-size: 15px;
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
  -webkit-text-size-adjust: none; }

body, select, input, textarea {
  color: #333333; }

a, a:active {
  color: #00451f; }

a:hover {
  color: black; }

input:-moz-placeholder {
  color: gray; }

textarea:-moz-placeholder {
  color: gray; }

li.separator {
  background: #eee;
  height: 1px;
  overflow: hidden; }

/*
 * Footer
 */
html, body {
  height: 100%; }

div.wrapper {
  min-height: 100%;
  height: auto !important;
  height: 100%;
  margin-bottom: -50px; }
  div.wrapper div.clear {
    height: 70px;
    clear: both; }

footer {
  height: 50px;
  width: 100%;
  background: #fff;
  color: #999;
  clear: both;
  padding: 0 15px 5px;
  overflow: hidden;
  position: fixed;
  bottom: 0; }
  footer form {
    position: relative;
    display: none; }
    footer form input {
      font-family: inherit;
      font-size: 1.1em;
      width: 100%;
      padding: 8px;
      padding-right: 30px; }
    footer form input.sending {
      background: url(../image/loader-facebook.gif) no-repeat 99% 50%; }
    footer form input.disabled, footer form input.sending.disabled {
      background-color: #eee; }
    footer form .menu {
      position: absolute;
      top: 5px;
      right: 4px; }
      footer form .menu a {
        color: #555;
        font-size: 22px;
        text-decoration: none;
        display: block;
        padding: 4px; }
  @media (max-width: 700px) {
    footer {
      position: static; } }

/*
 * Typographi and base CSS
 */
h1 a, h3 a {
  text-decoration: none; }

h1 {
  font-size: 2em;
  margin-top: 1em;
  margin-bottom: 1em; }

h2 {
  font-size: 1.6em;
  margin-top: 0.8em;
  margin-bottom: 0.8em; }

h3 {
  font-size: 1.2em;
  margin-top: 0.6em;
  margin-bottom: 0.6em; }

img {
  max-width: 100%; }

p {
  margin: 1em 0; }

ol {
  margin-left: 2em; }

dl dt {
  width: 220px;
  float: left;
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap; }
dl dd {
  margin-left: 240px; }

.center {
  text-align: center; }

.hidden {
  display: none; }

.row {
  padding: 0 20px;
  clear: both; }

.close {
  display: block;
  float: right;
  cursor: pointer;
  font-size: 1.3em;
  padding: 5px;
  line-height: 0.4em; }

div.question {
  padding: 0.5em;
  -moz-border-radius: 3px;
  -webkit-border-radius: 3px;
  border-radius: 3px; }

div.question, li.info {
  background: #def;
  color: #158;
  border: 1px solid #bde; }

div.question .button {
  background: #aad4ff;
  border: 2px solid #1188ff; }

div.question .button.confirm {
  background: #aad4ff;
  border: 2px solid #900; }

a.active {
  background-color: #e6e6e6; }

/*
 * Form elements
 */
form {
  margin: 0; }
  form label {
    display: block;
    margin-top: 20px; }
  form select[multi] {
    height: 30px; }
  form input[readonly] {
    background: #f6f6f6;
    color: #666; }
  form .selectize-input, form input, form select {
    background: #fff;
    font-size: 14px;
    outline: none;
    border: 2px solid #ccc;
    padding: 4px;
    width: 290px;
    -moz-border-radius: 3px;
    -webkit-border-radius: 3px;
    border-radius: 3px; }
  form .selectize-input {
    padding: 2px; }
  form .selectize-input.focus, form input:focus {
    border: 2px solid #a6a6a6; }
  form input[type="checkbox"] {
    width: auto; }
  form input.field-with-error, form select.field-with-error {
    border-color: #d5000d; }
  form div.actions {
    margin-top: 2em;
    clear: left;
    padding: 0.5em 0; }
  form p.help {
    color: #555;
    font-size: 0.9em;
    margin-top: 0.1em; }

form.horizontal label {
  margin-top: 4px;
  float: left;
  width: 8em; }
form.horizontal div.form-group {
  padding: 4px 0;
  clear: left; }
form.horizontal div.actions {
  margin-top: 1em;
  margin-left: 8em; }
form.horizontal p.help, form.horizontal p.error {
  font-size: 0.9em;
  margin: 2px 0 0 8.88889em; }
form.horizontal p.error {
  background: #ffa2a8;
  width: 290px;
  margin-top: -3px;
  padding: 4px;
  border: 2px solid #d5000d;
  border-top: 0;
  -moz-border-radius: 3px;
  -webkit-border-radius: 3px;
  border-radius: 3px; }
form.horizontal input.field-with-error, form.horizontal select.field-with-error {
  border-bottom: 0; }
@media (max-width: 400px) {
  form.horizontal label {
    float: none;
    width: 100%; }
  form.horizontal div.actions, form.horizontal p.help, form.horizontal p.error {
    margin-left: 0; } }

form.edit-mode a.edit-mode {
  display: none; }

button, .button {
  background: #fafafa;
  color: #222;
  font-size: 15px;
  text-decoration: none;
  padding: 3px 6px;
  border: 2px solid #999999;
  -moz-border-radius: 3px;
  -webkit-border-radius: 3px;
  border-radius: 3px; }

button {
  vertical-align: baseline; }

button:active, button.active, .button:active, .button.active {
  background: #cccccc; }

button.confirm, a.confirm {
  color: #900;
  border: 2px solid #900; }

button:hover, .button:hover {
  border: 2px solid #4d4d4d; }

hr {
  border: none;
  border-top: 1px solid #ddd;
  line-height: 1px;
  height: 1px;
  margin-top: 1em;
  padding-bottom: 1em; }

.selectize-input.dropdown-active {
  -moz-border-radius: 3px;
  -webkit-border-radius: 3px;
  border-radius: 3px; }

.selectize-input.dropdown-active::before {
  content: ' ';
  display: block;
  position: absolute;
  background: #f2f2f2;
  height: 1px;
  bottom: 0;
  left: 0;
  right: 0; }

.selectize-control.multi .selectize-input > div {
  background: #eee;
  margin-right: 5px;
  padding: 0 2px;
  cursor: pointer;
  -moz-border-radius: 3px;
  -webkit-border-radius: 3px;
  border-radius: 3px; }

.selectize-dropdown {
  background: #fafafa;
  border-top: 0 none;
  -webkit-box-shadow: 0px 2px 2px 2px rgba(0, 0, 0, 0.1);
  -moz-box-shadow: 0px 2px 2px 2px rgba(0, 0, 0, 0.1);
  box-shadow: 0px 2px 2px 2px rgba(0, 0, 0, 0.1); }

.selectize-dropdown-content {
  max-height: 200px;
  overflow-y: auto !important;
  overflow-x: hidden !important; }

.selectize-dropdown [data-selectable],
.selectize-dropdown .optgroup-header {
  padding: 6px 9px; }

.selectize-dropdown .optgroup:first-child .optgroup-header {
  border-top: 0 none; }

.selectize-dropdown .optgroup-header {
  background: #fafafa;
  border-bottom: 1px solid #e8e8e8;
  border-top: 1px solid #e8e8e8;
  font-weight: bold;
  font-size: 0.8em;
  cursor: default; }

.selectize-dropdown .create {
  color: #444; }

.selectize-dropdown .active {
  background: #f0f0f0; }

.selectize-dropdown [data-selectable] .highlight {
  background: rgba(255, 237, 40, 0.4);
  -moz-border-radius: 1px;
  -webkit-border-radius: 1px;
  border-radius: 1px; }

.selectize-input.disabled, .selectize-input.disabled * {
  cursor: default !important; }

/**********************************************************
* BASIC AESTHETIC STYLES (single)                        *
**********************************************************/
.selectize-control.single .selectize-input {
  cursor: pointer; }

.selectize-control.single .selectize-input::after {
  content: ' ';
  display: block;
  position: absolute;
  top: 50%;
  right: 15px;
  margin-top: -2px;
  width: 0;
  height: 0;
  border-style: solid;
  border-width: 5px 5px 0 5px;
  border-color: #808080 transparent transparent transparent; }

.selectize-control.single .selectize-input.dropdown-active::after {
  margin-top: -3px;
  border-width: 0 5px 5px 5px;
  border-color: transparent transparent #808080 transparent; }

.selectize-control.single .selectize-input.disabled {
  opacity: 0.5; }

/**********************************************************
* BASIC AESTHETIC STYLES (multi)                         *
**********************************************************/
.selectize-control.multi .selectize-input.disabled {
  background-color: #fafafa; }

/**********************************************************
* LAYOUT STYLES (mandatory)                              *
**********************************************************/
.selectize-control {
  position: relative; }

.selectize-input {
  overflow: hidden;
  position: relative;
  z-index: 1; }

.selectize-input:after {
  content: ' ';
  display: block;
  clear: left; }

.selectize-input .items {
  display: inline; }

.selectize-input > * {
  vertical-align: baseline;
  display: -moz-inline-stack;
  display: inline-block;
  zoom: 1;
  *display: inline; }

.selectize-input > input {
  max-width: 100% !important;
  text-indent: 0 !important;
  border: 0 none !important;
  background: none !important;
  padding: 0 !important;
  margin: 0;
  line-height: inherit !important;
  -webkit-user-select: auto !important;
  -webkit-box-shadow: none !important;
  -moz-box-shadow: none !important;
  box-shadow: none !important; }

.selectize-input > input:focus {
  outline: none !important; }

.selectize-dropdown {
  position: absolute;
  z-index: 2; }

.selectize-dropdown [data-selectable] {
  cursor: pointer;
  overflow: hidden; }

.selectize-control.plugin-drag_drop.multi > .selectize-input > div.ui-sortable-placeholder {
  visibility: visible !important;
  background: #f2f2f2 !important;
  background: rgba(0, 0, 0, 0.06) !important;
  border: 0 none !important;
  -webkit-box-shadow: inset 0 0 12px 4px white;
  -moz-box-shadow: inset 0 0 12px 4px white;
  box-shadow: inset 0 0 12px 4px white; }

.selectize-control.plugin-drag_drop .ui-sortable-placeholder::after {
  content: '!';
  visibility: hidden; }

.selectize-control.plugin-drag_drop .ui-sortable-helper {
  -webkit-box-shadow: 0 2px 5px rgba(0, 0, 0, 0.2) !important;
  -moz-box-shadow: 0 2px 5px rgba(0, 0, 0, 0.2) !important;
  box-shadow: 0 2px 5px rgba(0, 0, 0, 0.2) !important; }

.selectize-dropdown-header {
  padding: 6px 9px;
  -moz-border-radius: 3px 3px 0 0;
  -webkit-border-radius: 3px 3px 0 0;
  border-radius: 3px 3px 0 0; }

.selectize-dropdown-header-close {
  float: right;
  color: #ddd;
  font-size: 20px !important; }

.selectize-dropdown-header-close:hover {
  color: #aaa; }

.selectize-control.plugin-optgroup_columns .optgroup {
  border-right: 1px solid #f2f2f2;
  float: left; }

.selectize-control.plugin-optgroup_columns .optgroup:last-child {
  border-right: 0 none; }

.selectize-control.plugin-optgroup_columns .optgroup-header {
  border-top: 0 none; }

.selectize-control.plugin-remove_button .item {
  position: relative;
  padding-right: 24px !important; }

.selectize-control.plugin-remove_button .item .remove {
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  width: 17px;
  text-align: center;
  font-size: 12px;
  font-weight: bold;
  color: inherit;
  vertical-align: middle;
  display: inline-block;
  padding: 7px 0 0 0;
  line-height: 8px;
  border-left: 1px solid #74b21e;
  -moz-border-radius: 0 2px 2px 0;
  -webkit-border-radius: 0 2px 2px 0;
  border-radius: 0 2px 2px 0; }

.selectize-control.plugin-remove_button .item .remove:hover {
  border-left-color: #5e8f1a;
  background: rgba(50, 90, 0, 0.15); }

/*
 * Loading indicator
 */
div.loading-indicator-overlay,
div.loading-indicator {
  height: 100%;
  width: 100%;
  position: fixed;
  top: 0;
  left: 0;
  z-index: 900; }

div.loading-indicator-overlay {
  background: #fff;
  opacity: 0.7;
  -ms-filter: "progid:DXImageTransform.Microsoft.Alpha(Opacity=" 70 ")";
  filter: alpha(opacity=70);
  zoom: 1; }

div.loading-indicator {
  background: url(../image/loader-squares-circle.gif) 50% 50% no-repeat; }

/*
 * Chat
 */
nav.bar {
  background: #fafafa;
  position: fixed;
  padding: 0 20px;
  top: 0;
  left: 0;
  right: 0;
  height: 42px;
  z-index: 1000;
  overflow: hidden;
  -webkit-box-shadow: 0 4px 4px -2px rgba(180, 180, 180, 0.5);
  -moz-box-shadow: 0 4px 4px -2px rgba(180, 180, 180, 0.5);
  box-shadow: 0 4px 4px -2px rgba(180, 180, 180, 0.5);
  opacity: 0.95;
  -ms-filter: "progid:DXImageTransform.Microsoft.Alpha(Opacity=" 95 ")";
  filter: alpha(opacity=95);
  zoom: 1; }
  nav.bar a, nav.bar a:active {
    color: #000;
    text-decoration: none;
    padding: 10px 4px;
    display: block;
    float: left; }
  nav.bar a.conversation {
    color: #666; }
  nav.bar a.conversations.toggler {
    display: none; }
  nav.bar a.sidebar.toggler {
    display: none; }
  nav.bar .right {
    background: #fafafa;
    position: absolute;
    right: 20px; }
    nav.bar .right a, nav.bar .right a:active {
      font-size: 1.2em;
      width: 2em;
      text-align: center; }
    nav.bar .right a.notifications.toggler {
      position: relative; }
    nav.bar .right a.notifications.toggler b {
      position: absolute;
      bottom: -3px;
      right: 2px;
      color: #bbb;
      font-size: 0.9em;
      font-weight: normal; }
    nav.bar .right a.notifications.toggler.alert b {
      color: #b22; }
    nav.bar .right a.hidden {
      display: none; }
  nav.bar a.active, nav.bar a:hover, nav.bar a:active {
    background-color: #d9d9d9;
    color: #000; }
  nav.bar ul.conversations li.unread a {
    color: #aa0000; }

div.notification.question {
  padding-left: 20px;
  width: 100%;
  z-index: 999;
  position: fixed;
  top: 40px;
  display: none; }
  div.notification.question button, div.notification.question .button {
    font-size: 0.9em; }

div.add-conversation.container,
div.conversations.container,
div.notifications.container,
div.sidebar.container {
  background: #f8f8f8;
  position: fixed;
  top: 39px;
  width: 300px;
  overflow: hidden;
  overflow-y: auto;
  -webkit-overflow-scrolling: touch;
  z-index: 1001;
  display: none;
  border: 1px solid #d9d9d9;
  border-top-width: 6px; }
  div.add-conversation.container h3, div.add-conversation.container a,
  div.conversations.container h3,
  div.conversations.container a,
  div.notifications.container h3,
  div.notifications.container a,
  div.sidebar.container h3,
  div.sidebar.container a {
    color: #000;
    text-decoration: none;
    padding: 4px;
    display: block; }
  div.add-conversation.container h3,
  div.conversations.container h3,
  div.notifications.container h3,
  div.sidebar.container h3 {
    font-size: 1em;
    margin: 0; }
  div.add-conversation.container button.inactive, div.add-conversation.container a.inactive,
  div.conversations.container button.inactive,
  div.conversations.container a.inactive,
  div.notifications.container button.inactive,
  div.notifications.container a.inactive,
  div.sidebar.container button.inactive,
  div.sidebar.container a.inactive {
    color: #999999; }
  div.add-conversation.container ul ul li a, div.add-conversation.container ul ul li button,
  div.conversations.container ul ul li a,
  div.conversations.container ul ul li button,
  div.notifications.container ul ul li a,
  div.notifications.container ul ul li button,
  div.sidebar.container ul ul li a,
  div.sidebar.container ul ul li button {
    background-image: url(../image/fa-caret-right.png);
    background-position: 9px 50%;
    background-repeat: no-repeat;
    font-size: 0.9em;
    height: 2em;
    padding: 2px 2px 2px 20px;
    border: 0; }
  div.add-conversation.container a:hover, div.add-conversation.container a:active, div.add-conversation.container a:focus,
  div.conversations.container a:hover,
  div.conversations.container a:active,
  div.conversations.container a:focus,
  div.notifications.container a:hover,
  div.notifications.container a:active,
  div.notifications.container a:focus,
  div.sidebar.container a:hover,
  div.sidebar.container a:active,
  div.sidebar.container a:focus {
    background-color: #e1e1e1;
    color: #000;
    outline: 0; }
  div.add-conversation.container a:focus, div.add-conversation.container button:focus,
  div.conversations.container a:focus,
  div.conversations.container button:focus,
  div.notifications.container a:focus,
  div.notifications.container button:focus,
  div.sidebar.container a:focus,
  div.sidebar.container button:focus {
    background-color: #ebebeb; }

div.sidebar.container button {
  background: transparent;
  width: 100%;
  text-align: left;
  display: block;
  overflow: visible;
  /* shrinkwrap the text in <IE7 */
  cursor: pointer;
  -moz-user-select: text; }
div.sidebar.container button:hover, div.sidebar.container button:focus {
  background-color: #e1e1e1;
  color: #000; }

div.add-conversation.container {
  padding-left: 3px;
  padding-bottom: 6px;
  height: auto; }

div.conversations.container li.unread a {
  color: #aa0000; }

div.notifications.container li.unread {
  border-right: 2px solid #f66; }
div.notifications.container span.message {
  color: #666;
  font-size: 0.9em;
  line-height: 1em;
  text-overflow: ellipsis;
  white-space: nowrap;
  display: block;
  overflow: hidden; }

@media (min-width: 700px) {
  div.sidebar.container {
    background: #f3f3f3;
    border: 0;
    border-left: 1px solid #ddd;
    width: 180px;
    position: fixed;
    display: block;
    top: 42px;
    bottom: -1px;
    right: 0;
    z-index: 2; }
    div.sidebar.container h3, div.sidebar.container a {
      text-decoration: none;
      display: block;
      padding: 4px 8px; } }
div.messages {
  padding-top: 45px; }
  div.messages li {
    display: block;
    position: relative;
    padding: 3px 0;
    overflow: hidden; }
    div.messages li div.question {
      padding-left: 50px; }
    div.messages li button, div.messages li .button {
      font-size: 0.9em; }
    div.messages li img.avatar {
      float: left;
      max-width: 40px; }
    div.messages li h3 {
      font-size: 1em;
      color: #467;
      line-height: 1.2em;
      margin: 0; }
    div.messages li h3, div.messages li div.content, div.messages li div.embed {
      margin-left: 50px; }
    div.messages li div.content {
      max-width: 40em;
      line-height: 1.4em; }
    div.messages li div.whitespace {
      white-space: pre-wrap; }
    div.messages li div.embed {
      max-height: 220px; }
      div.messages li div.embed img {
        max-height: 220px;
        border: 2px solid #bbb;
        -moz-border-radius: 20px;
        -webkit-border-radius: 20px;
        border-radius: 20px;
        margin: 8px 0; }
    div.messages li span.timestamp {
      position: absolute;
      top: 2px;
      right: 2px;
      color: #999;
      font-size: .85em; }
    div.messages li span.actions {
      float: right;
      padding-right: 2px; }
  div.messages li.same-nick {
    padding-top: 0; }
  div.messages li.action.message div.content {
    padding-top: 20px; }
  div.messages li.action.message.same-nick div.content {
    padding-top: 0; }
  div.messages li.action.message div.content a {
    font-weight: normal;
    text-decoration: none; }
  div.messages li.message-pending div.content {
    color: #999; }
  div.messages li.message-pending h3 {
    color: #ccc; }
  div.messages li.message-error {
    font-style: italic; }
  div.messages li.message-error h3 {
    color: #333;
    font-style: italic; }
  div.messages li.message-error, div.messages li.network-message.error {
    background: #ffa2a8; }
  div.messages li.nick-change, div.messages li.nick-joined, div.messages li.nick-parted, div.messages li.nick-quit, div.messages li.notice, div.messages li.topic, div.messages li.whois {
    background-color: #f9f7fa;
    color: #888; }
  div.messages li.nick-change .content b, div.messages li.nick-parted .content b, div.messages li.nick-quit .content b {
    font-weight: normal; }

body.chat footer form {
  display: block; }
body.chat ul.conversations li.first a {
  background-color: #f2f2f2;
  font-weight: bold;
  color: #000; }

body.convos a.convos {
  background-color: #f2f2f2; }
body.convos footer form {
  display: block; }

body.without-sidebar div.sidebar.container {
  display: none; }

body.with-sidebar footer form {
  margin-right: 180px; }
body.with-sidebar div.messages {
  margin-right: 180px; }
  body.with-sidebar div.messages li.highlight {
    background: #fee; }

body.historic div.sidebar.container {
  display: none; }
body.historic footer {
  display: none; }
body.historic div.clear {
  height: 20px; }

div.goto-bottom a {
  display: none;
  background: #fafafa;
  color: #333;
  position: fixed;
  right: 6px;
  bottom: 42px;
  padding: 5px;
  z-index: 1000;
  border: 1px solid #ccc;
  -moz-border-radius: 4px;
  -webkit-border-radius: 4px;
  border-radius: 4px;
  opacity: 0.8;
  -ms-filter: "progid:DXImageTransform.Microsoft.Alpha(Opacity=" 80 ")";
  filter: alpha(opacity=80);
  zoom: 1; }

body.tactile {
  background: #e7e7e7 url(../image/tactile-body-bg.png) 0 0 repeat; }

/*
 * Landing page
 */
.landing-page {
  background: transparent url(../image/tactile-highlight-bg.jpg) 50% 0 no-repeat;
  min-height: 595px; }
  .landing-page .row {
    margin: auto;
    padding-top: 1em;
    width: 680px; }
  .landing-page h1 {
    margin-bottom: 0; }
    .landing-page h1 .logo {
      text-indent: -1000px;
      display: block;
      height: 55px;
      width: 300px;
      background: url("/image/logo.png") no-repeat top left;
      background: none, url("/image/logo.svg") no-repeat top left; }
    .landing-page h1 small {
      display: block;
      font-weight: normal;
      color: #999; }
  .landing-page h2.login-register {
    font-size: 1.1em; }

div.footer {
  font-size: 0.8em;
  margin-top: 2em;
  padding-top: 1em; }
  div.footer a {
    color: #777;
    text-decoration: none; }

@media (max-width: 700px) {
  footer, nav.bar, .row {
    padding: 0; }

  nav.bar .right {
    right: 0; }

  div.messages {
    margin-right: 0; }

  body.with-sidebar nav.bar a.sidebar.toggler {
    display: block; }
  body.with-sidebar div.messages, body.with-sidebar footer form {
    margin-right: 0; }
  body.with-sidebar div.sidebar.container {
    display: none; }

  body.with-sidebar div.wrapper,
  body.without-sidebar div.wrapper {
    margin-bottom: -42px; }
    body.with-sidebar div.wrapper div.clear,
    body.without-sidebar div.wrapper div.clear {
      height: 42px; }

  footer {
    height: 42px; }
    footer input, footer input:focus {
      border-bottom: 0;
      border-right: 0;
      border-left: 0; }

  .landing-page img {
    margin: 2em 0; }
  .landing-page nav, .landing-page .login, .landing-page .register, .landing-page .about {
    width: 100%; }
  .landing-page .row {
    width: 100%;
    padding-left: 10px;
    padding-right: 10px; }

  dl dt {
    float: none;
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap; }
  dl dd {
    margin-left: 20px; } }
@font-face {
  font-family: 'FontAwesome';
  src: url("../font/fontawesome-webfont.eot?v=3.0.1");
  src: url("../font/fontawesome-webfont.eot?#iefix&v=3.0.1") format("embedded-opentype"), url("../font/fontawesome-webfont.woff?v=3.0.1") format("woff"), url("../font/fontawesome-webfont.ttf?v=3.0.1") format("truetype");
  font-weight: normal;
  font-style: normal; }

[class^="icon-"],
[class*=" icon-"] {
  font-family: FontAwesome;
  font-weight: normal;
  font-style: normal;
  text-decoration: inherit;
  -webkit-font-smoothing: antialiased;
  display: inline;
  width: auto;
  height: auto;
  line-height: normal;
  vertical-align: baseline;
  background-image: none;
  background-position: 0% 0%;
  background-repeat: repeat;
  margin-top: 0; }

[class^="icon-"]:before,
[class*=" icon-"]:before {
  text-decoration: inherit;
  display: inline-block;
  speak: none; }

/* makes sure icons active on rollover in links */
a [class^="icon-"],
a [class*=" icon-"] {
  display: inline-block; }

/* makes the font 33% larger relative to the icon container */
.icon-large:before {
  vertical-align: -10%;
  font-size: 4/3em; }

.icon-border {
  border: solid 1px #eeeeee;
  padding: .2em .25em .15em;
  -moz-border-radius: 3px;
  -webkit-border-radius: 3px;
  border-radius: 3px; }

[class^="icon-"].pull-left,
[class*=" icon-"].pull-left {
  margin-right: .3em; }
[class^="icon-"].pull-right,
[class*=" icon-"].pull-right {
  margin-left: .3em; }

/*  Font Awesome uses the Unicode Private Use Area (PUA) to ensure screen
    readers do not read off random characters that represent icons */
.icon-info-sign:before {
  content: "\f023"; }

.icon-chevron-down:before {
  content: "\f024"; }

.icon-comments:before {
  content: "\f025"; }

.icon-bullhorn:before {
  content: "\f027"; }

.icon-signout:before {
  content: "\f026"; }

.icon-user:before {
  content: "\f021"; }

.icon-cog:before {
  content: "\f022"; }

body.chat nav .icon-cog:before {
  content: "\f021"; }
